@@ -156,7 +156,7 @@ static void hso_gprs_activate_primary(struct ofono_gprs_context *gc,
 	char buf[AUTH_BUF_LENGTH];
 	int len;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	gcd->active_context = ctx->cid;
@@ -165,7 +165,7 @@ static void hso_gprs_activate_primary(struct ofono_gprs_context *gc,
 
 	if (ctx->username[0] && ctx->password[0])
 		snprintf(buf, sizeof(buf), "AT$QCPDPP=%u,1,\"%s\",\"%s\"",
-			ctx->cid, ctx->username, ctx->password);
+			ctx->cid, ctx->password, ctx->username);
 	else if (ctx->password[0])
 		snprintf(buf, sizeof(buf), "AT$QCPDPP=%u,2,,\"%s\"",
 				ctx->cid, ctx->password);
@@ -200,7 +200,7 @@ static void hso_gprs_deactivate_primary(struct ofono_gprs_context *gc,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = gc;
@@ -389,12 +389,12 @@ static struct ofono_gprs_context_driver driver = {
 	.deactivate_primary	= hso_gprs_deactivate_primary,
 };
 
-void hso_gprs_context_init()
+void hso_gprs_context_init(void)
 {
 	ofono_gprs_context_driver_register(&driver);
 }
 
-void hso_gprs_context_exit()
+void hso_gprs_context_exit(void)
 {
 	ofono_gprs_context_driver_unregister(&driver);
 }
