@@ -119,13 +119,11 @@ static struct isi_call_req_ctx *isi_call_req(struct ofono_voicecall *ovc,
 	irc->cb = cb;
 	irc->data = data;
 
-	if (g_isi_client_send(ivc->client, req, len, ISI_CALL_TIMEOUT,
-				handler, irc, NULL) == NULL) {
-		g_free(irc);
-		return NULL;
-	}
+	if (g_isi_client_send(ivc->client, req, len, handler, irc, NULL))
+		return irc;
 
-	return irc;
+	g_free(irc);
+	return NULL;
 }
 
 static void isi_ctx_queue(struct isi_call_req_ctx *irc, isi_call_req_step *next)
@@ -291,7 +289,7 @@ static struct isi_call *isi_call_status_info_sb_proc(struct isi_voicecall *ivc,
 	struct call_info *ci;
 	size_t len = sizeof(struct call_info);
 
-	if (!g_isi_sb_iter_get_struct(sb, (void *)&ci, len, 2))
+	if (!g_isi_sb_iter_get_struct(sb, (void *) &ci, len, 2))
 		return NULL;
 
 	i = ci->call_id & 7;
@@ -317,7 +315,7 @@ static struct isi_call *isi_call_addr_and_status_info_sb_proc(
 	size_t len = sizeof(struct call_addr_info);
 	char *addr;
 
-	if (!g_isi_sb_iter_get_struct(sb, (void *)&ci, len, 2))
+	if (!g_isi_sb_iter_get_struct(sb, (void *) &ci, len, 2))
 		return NULL;
 
 	if (!g_isi_sb_iter_get_alpha_tag(sb, &addr, 2 * ci->addr_len, 12))
@@ -797,7 +795,7 @@ static void isi_call_control_resp(const GIsiMessage *msg, void *data)
 	}
 
 	if (cause == CALL_CAUSE_NO_CAUSE) {
-		isi_ctx_return_failure(irc);
+		isi_ctx_return_success(irc);
 		return;
 	}
 
@@ -867,7 +865,7 @@ static void isi_call_dtmf_send_resp(const GIsiMessage *msg, void *data)
 	struct isi_call_req_ctx *irc = data;
 	GIsiSubBlockIter iter;
 	uint8_t cause_type;
-	uint8_t cause;
+	uint8_t cause = CALL_CAUSE_NO_CAUSE;
 
 	if (!check_response_status(msg, CALL_DTMF_SEND_RESP))
 		goto error;
@@ -929,9 +927,8 @@ static struct isi_call_req_ctx *isi_call_dtmf_send_req(struct ofono_voicecall *o
 
 static void isi_dial(struct ofono_voicecall *ovc,
 			const struct ofono_phone_number *restrict number,
-			enum ofono_clir_option clir,
-			enum ofono_cug_option cug,
-			ofono_voicecall_cb_t cb, void *data)
+			enum ofono_clir_option clir, ofono_voicecall_cb_t cb,
+			void *data)
 {
 	unsigned char presentation = CALL_GSM_PRESENTATION_DEFAULT;
 
@@ -947,15 +944,6 @@ static void isi_dial(struct ofono_voicecall *ovc,
 		break;
 	}
 
-	switch (cug) {
-	case OFONO_CUG_OPTION_DEFAULT:
-		break;
-	case OFONO_CUG_OPTION_INVOCATION:
-		/* Not implemented */
-		CALLBACK_WITH_FAILURE(cb, data);
-		return;
-	}
-
 	isi_call_create_req(ovc, presentation, number->type, number->number,
 				cb, data);
 }
@@ -975,6 +963,7 @@ static void isi_hangup_current(struct ofono_voicecall *ovc,
 	 */
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
 	int id = 0;
+	uint8_t cause = CALL_CAUSE_RELEASE_BY_USER;
 
 	for (id = 1; id <= 7; id++) {
 		if (ivc->calls[id].call_id & CALL_ID_WAITING)
@@ -985,19 +974,23 @@ static void isi_hangup_current(struct ofono_voicecall *ovc,
 		switch (ivc->calls[id].status) {
 		case CALL_STATUS_CREATE:
 		case CALL_STATUS_COMING:
-		case CALL_STATUS_PROCEEDING:
 		case CALL_STATUS_MO_ALERTING:
-		case CALL_STATUS_MT_ALERTING:
 		case CALL_STATUS_ANSWERED:
 			goto release_by_id;
+		case CALL_STATUS_MT_ALERTING:
+			cause = CALL_CAUSE_BUSY_USER_REQUEST;
+			goto release_by_id;
+		case CALL_STATUS_PROCEEDING:
+			if (ivc->calls[id].mode_info & CALL_MODE_ORIGINATOR)
+				cause = CALL_CAUSE_BUSY_USER_REQUEST;
+			goto release_by_id;
 		}
 	}
 
 	id = CALL_ID_ACTIVE;
 
 release_by_id:
-	isi_call_release_req(ovc, id, CALL_CAUSE_TYPE_CLIENT,
-				CALL_CAUSE_RELEASE_BY_USER, cb, data);
+	isi_call_release_req(ovc, id, CALL_CAUSE_TYPE_CLIENT, cause, cb, data);
 }
 
 static void isi_release_all_held(struct ofono_voicecall *ovc,
@@ -1024,7 +1017,7 @@ static void isi_retrieve(struct ofono_voicecall *ovc,
 
 static void isi_wait_and_answer(struct isi_call_req_ctx *irc, int event)
 {
-	DBG("irc=%p event=%u", (void *)irc, event);
+	DBG("irc=%p event=%u", (void *) irc, event);
 
 	if (event != CALL_STATUS_TERMINATED)
 		return;
@@ -1035,7 +1028,7 @@ static void isi_wait_and_answer(struct isi_call_req_ctx *irc, int event)
 
 static void isi_wait_and_retrieve(struct isi_call_req_ctx *irc, int event)
 {
-	DBG("irc=%p event=%u", (void *)irc, event);
+	DBG("irc=%p event=%u", (void *) irc, event);
 
 	if (event != CALL_STATUS_TERMINATED)
 		return;
@@ -1330,12 +1323,12 @@ static struct ofono_voicecall_driver driver = {
 	.send_tones		= isi_send_tones,
 };
 
-void isi_voicecall_init()
+void isi_voicecall_init(void)
 {
 	ofono_voicecall_driver_register(&driver);
 }
 
-void isi_voicecall_exit()
+void isi_voicecall_exit(void)
 {
 	ofono_voicecall_driver_unregister(&driver);
 }
